@@ -18,7 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 // Qt
-#include <QtTest/QtTest>
+#include <QtTest>
 // KWin
 #include "../../src/client/connection_thread.h"
 #include "../../src/client/event_queue.h"
@@ -53,6 +53,9 @@ private Q_SLOTS:
     void testDpms_data();
     void testDpms();
 
+    void testDpmsRequestMode_data();
+    void testDpmsRequestMode();
+
 private:
     KWayland::Server::Display *m_display;
     KWayland::Server::OutputInterface *m_serverOutput;
@@ -82,10 +85,16 @@ void TestWaylandOutput::init()
     QVERIFY(m_display->isRunning());
 
     m_serverOutput = m_display->createOutput(this);
+    QCOMPARE(m_serverOutput->pixelSize(), QSize());
+    QCOMPARE(m_serverOutput->refreshRate(), 60000);
     m_serverOutput->addMode(QSize(800, 600), OutputInterface::ModeFlags(OutputInterface::ModeFlag::Preferred));
+    QCOMPARE(m_serverOutput->pixelSize(), QSize(800, 600));
     m_serverOutput->addMode(QSize(1024, 768));
     m_serverOutput->addMode(QSize(1280, 1024), OutputInterface::ModeFlags(), 90000);
+    QCOMPARE(m_serverOutput->pixelSize(), QSize(800, 600));
     m_serverOutput->setCurrentMode(QSize(1024, 768));
+    QCOMPARE(m_serverOutput->pixelSize(), QSize(1024, 768));
+    QCOMPARE(m_serverOutput->refreshRate(), 60000);
     m_serverOutput->create();
     QCOMPARE(m_serverOutput->isDpmsSupported(), false);
     QCOMPARE(m_serverOutput->dpmsMode(), OutputInterface::DpmsMode::On);
@@ -132,8 +141,25 @@ void TestWaylandOutput::cleanup()
 
 void TestWaylandOutput::testRegistry()
 {
+    QSignalSpy globalPositionChangedSpy(m_serverOutput, &KWayland::Server::OutputInterface::globalPositionChanged);
+    QVERIFY(globalPositionChangedSpy.isValid());
+    QCOMPARE(m_serverOutput->globalPosition(), QPoint(0, 0));
     m_serverOutput->setGlobalPosition(QPoint(100, 50));
+    QCOMPARE(m_serverOutput->globalPosition(), QPoint(100, 50));
+    QCOMPARE(globalPositionChangedSpy.count(), 1);
+    // changing again should not trigger signal
+    m_serverOutput->setGlobalPosition(QPoint(100, 50));
+    QCOMPARE(globalPositionChangedSpy.count(), 1);
+
+    QSignalSpy physicalSizeChangedSpy(m_serverOutput, &KWayland::Server::OutputInterface::physicalSizeChanged);
+    QVERIFY(physicalSizeChangedSpy.isValid());
+    QCOMPARE(m_serverOutput->physicalSize(), QSize());
     m_serverOutput->setPhysicalSize(QSize(200, 100));
+    QCOMPARE(m_serverOutput->physicalSize(), QSize(200, 100));
+    QCOMPARE(physicalSizeChangedSpy.count(), 1);
+    // changing again should not trigger signal
+    m_serverOutput->setPhysicalSize(QSize(200, 100));
+    QCOMPARE(physicalSizeChangedSpy.count(), 1);
 
     KWayland::Client::Registry registry;
     QSignalSpy announced(&registry, SIGNAL(outputAnnounced(quint32,quint32)));
@@ -159,7 +185,10 @@ void TestWaylandOutput::testRegistry()
     QSignalSpy outputChanged(&output, SIGNAL(changed()));
     QVERIFY(outputChanged.isValid());
 
-    output.setup(registry.bindOutput(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
+    auto o = registry.bindOutput(announced.first().first().value<quint32>(), announced.first().last().value<quint32>());
+    QVERIFY(!KWayland::Client::Output::get(o));
+    output.setup(o);
+    QCOMPARE(KWayland::Client::Output::get(o), &output);
     wl_display_flush(m_connection->display());
     QVERIFY(outputChanged.wait());
 
@@ -179,6 +208,20 @@ void TestWaylandOutput::testRegistry()
 
 void TestWaylandOutput::testModeChanges()
 {
+    // verify the server modes
+    using namespace KWayland::Server;
+    const auto serverModes = m_serverOutput->modes();
+    QCOMPARE(serverModes.count(), 3);
+    QCOMPARE(serverModes.at(0).size, QSize(800, 600));
+    QCOMPARE(serverModes.at(1).size, QSize(1024, 768));
+    QCOMPARE(serverModes.at(2).size, QSize(1280, 1024));
+    QCOMPARE(serverModes.at(0).refreshRate, 60000);
+    QCOMPARE(serverModes.at(1).refreshRate, 60000);
+    QCOMPARE(serverModes.at(2).refreshRate, 90000);
+    QCOMPARE(serverModes.at(0).flags, OutputInterface::ModeFlags(OutputInterface::ModeFlag::Preferred));
+    QCOMPARE(serverModes.at(1).flags, OutputInterface::ModeFlags(OutputInterface::ModeFlag::Current));
+    QCOMPARE(serverModes.at(2).flags, OutputInterface::ModeFlags());
+
     using namespace KWayland::Client;
     KWayland::Client::Registry registry;
     QSignalSpy announced(&registry, SIGNAL(outputAnnounced(quint32,quint32)));
@@ -253,6 +296,7 @@ void TestWaylandOutput::testModeChanges()
     outputChanged.clear();
     modeChangedSpy.clear();
     m_serverOutput->setCurrentMode(QSize(1280, 1024), 90000);
+    QCOMPARE(m_serverOutput->refreshRate(), 90000);
     QVERIFY(modeChangedSpy.wait());
     if (modeChangedSpy.size() == 1) {
         QVERIFY(modeChangedSpy.wait());
@@ -290,9 +334,18 @@ void TestWaylandOutput::testScaleChange()
 
     // change the scale
     outputChanged.clear();
+    QCOMPARE(m_serverOutput->scale(), 1);
+    QSignalSpy serverScaleChanged(m_serverOutput, &KWayland::Server::OutputInterface::scaleChanged);
+    QVERIFY(serverScaleChanged.isValid());
     m_serverOutput->setScale(2);
+    QCOMPARE(m_serverOutput->scale(), 2);
+    QCOMPARE(serverScaleChanged.count(), 1);
     QVERIFY(outputChanged.wait());
     QCOMPARE(output.scale(), 2);
+    // changing to same value should not trigger
+    m_serverOutput->setScale(2);
+    QCOMPARE(serverScaleChanged.count(), 1);
+    QVERIFY(!outputChanged.wait(100));
 
     // change once more
     outputChanged.clear();
@@ -320,7 +373,15 @@ void TestWaylandOutput::testSubPixel()
     using namespace KWayland::Client;
     using namespace KWayland::Server;
     QFETCH(OutputInterface::SubPixel, actual);
+    QCOMPARE(m_serverOutput->subPixel(), OutputInterface::SubPixel::Unknown);
+    QSignalSpy serverSubPixelChangedSpy(m_serverOutput, &OutputInterface::subPixelChanged);
+    QVERIFY(serverSubPixelChangedSpy.isValid());
     m_serverOutput->setSubPixel(actual);
+    QCOMPARE(m_serverOutput->subPixel(), actual);
+    QCOMPARE(serverSubPixelChangedSpy.count(), 1);
+    // changing to same value should not trigger the signal
+    m_serverOutput->setSubPixel(actual);
+    QCOMPARE(serverSubPixelChangedSpy.count(), 1);
 
     KWayland::Client::Registry registry;
     QSignalSpy announced(&registry, SIGNAL(outputAnnounced(quint32,quint32)));
@@ -344,6 +405,8 @@ void TestWaylandOutput::testSubPixel()
     // change back to unknown
     outputChanged.clear();
     m_serverOutput->setSubPixel(OutputInterface::SubPixel::Unknown);
+    QCOMPARE(m_serverOutput->subPixel(), OutputInterface::SubPixel::Unknown);
+    QCOMPARE(serverSubPixelChangedSpy.count(), 2);
     if (outputChanged.isEmpty()) {
         QVERIFY(outputChanged.wait());
     }
@@ -371,7 +434,15 @@ void TestWaylandOutput::testTransform()
     using namespace KWayland::Client;
     using namespace KWayland::Server;
     QFETCH(OutputInterface::Transform, actual);
+    QCOMPARE(m_serverOutput->transform(), OutputInterface::Transform::Normal);
+    QSignalSpy serverTransformChangedSpy(m_serverOutput, &OutputInterface::transformChanged);
+    QVERIFY(serverTransformChangedSpy.isValid());
     m_serverOutput->setTransform(actual);
+    QCOMPARE(m_serverOutput->transform(), actual);
+    QCOMPARE(serverTransformChangedSpy.count(), 1);
+    // changing to same should not trigger signal
+    m_serverOutput->setTransform(actual);
+    QCOMPARE(serverTransformChangedSpy.count(), 1);
 
     KWayland::Client::Registry registry;
     QSignalSpy announced(&registry, SIGNAL(outputAnnounced(quint32,quint32)));
@@ -394,6 +465,8 @@ void TestWaylandOutput::testTransform()
     // change back to normal
     outputChanged.clear();
     m_serverOutput->setTransform(OutputInterface::Transform::Normal);
+    QCOMPARE(m_serverOutput->transform(), OutputInterface::Transform::Normal);
+    QCOMPARE(serverTransformChangedSpy.count(), 2);
     if (outputChanged.isEmpty()) {
         QVERIFY(outputChanged.wait());
     }
@@ -472,6 +545,84 @@ void TestWaylandOutput::testDpms()
     QVERIFY(clientDpmsModeChangedSpy.wait());
     QCOMPARE(clientDpmsModeChangedSpy.count(), 1);
     QTEST(dpms->mode(), "client");
+
+    // test supported changed
+    QSignalSpy supportedChangedSpy(dpms, &Dpms::supportedChanged);
+    QVERIFY(supportedChangedSpy.isValid());
+    m_serverOutput->setDpmsSupported(false);
+    QVERIFY(supportedChangedSpy.wait());
+    QCOMPARE(supportedChangedSpy.count(), 1);
+    QVERIFY(!dpms->isSupported());
+    m_serverOutput->setDpmsSupported(true);
+    QVERIFY(supportedChangedSpy.wait());
+    QCOMPARE(supportedChangedSpy.count(), 2);
+    QVERIFY(dpms->isSupported());
+
+    // and switch back to on
+    m_serverOutput->setDpmsMode(OutputInterface::DpmsMode::On);
+    QVERIFY(clientDpmsModeChangedSpy.wait());
+    QCOMPARE(clientDpmsModeChangedSpy.count(), 2);
+    QCOMPARE(dpms->mode(), Dpms::Mode::On);
+}
+
+void TestWaylandOutput::testDpmsRequestMode_data()
+{
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+
+    QTest::addColumn<KWayland::Client::Dpms::Mode>("client");
+    QTest::addColumn<KWayland::Server::OutputInterface::DpmsMode>("server");
+
+    QTest::newRow("Standby") << Dpms::Mode::Standby << OutputInterface::DpmsMode::Standby;
+    QTest::newRow("Suspend") << Dpms::Mode::Suspend << OutputInterface::DpmsMode::Suspend;
+    QTest::newRow("Off") << Dpms::Mode::Off << OutputInterface::DpmsMode::Off;
+    QTest::newRow("On") << Dpms::Mode::On << OutputInterface::DpmsMode::On;
+}
+
+void TestWaylandOutput::testDpmsRequestMode()
+{
+    // this test verifies that requesting a dpms change from client side emits the signal on server side
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+
+    // setup code
+    m_display->createDpmsManager()->create();
+
+    // set Dpms on the Output
+    QSignalSpy serverDpmsSupportedChangedSpy(m_serverOutput, &OutputInterface::dpmsSupportedChanged);
+    QVERIFY(serverDpmsSupportedChangedSpy.isValid());
+    QCOMPARE(m_serverOutput->isDpmsSupported(), false);
+    m_serverOutput->setDpmsSupported(true);
+    QCOMPARE(serverDpmsSupportedChangedSpy.count(), 1);
+    QCOMPARE(m_serverOutput->isDpmsSupported(), true);
+
+    KWayland::Client::Registry registry;
+    registry.setEventQueue(m_queue);
+    QSignalSpy announced(&registry, &Registry::interfacesAnnounced);
+    QVERIFY(announced.isValid());
+    QSignalSpy dpmsAnnouncedSpy(&registry, &Registry::dpmsAnnounced);
+    QVERIFY(dpmsAnnouncedSpy.isValid());
+    registry.create(m_connection->display());
+    QVERIFY(registry.isValid());
+    registry.setup();
+    m_connection->flush();
+    QVERIFY(announced.wait());
+    QCOMPARE(dpmsAnnouncedSpy.count(), 1);
+
+    Output *output = registry.createOutput(registry.interface(Registry::Interface::Output).name, registry.interface(Registry::Interface::Output).version, &registry);
+
+    DpmsManager *dpmsManager = registry.createDpmsManager(dpmsAnnouncedSpy.first().first().value<quint32>(), dpmsAnnouncedSpy.first().last().value<quint32>(), &registry);
+    QVERIFY(dpmsManager->isValid());
+
+    Dpms *dpms = dpmsManager->getDpms(output, &registry);
+    // and test request mode
+    QSignalSpy modeRequestedSpy(m_serverOutput, &OutputInterface::dpmsModeRequested);
+    QVERIFY(modeRequestedSpy.isValid());
+
+    QFETCH(Dpms::Mode, client);
+    dpms->requestMode(client);
+    QVERIFY(modeRequestedSpy.wait());
+    QTEST(modeRequestedSpy.last().first().value<OutputInterface::DpmsMode>(), "server");
 }
 
 QTEST_GUILESS_MAIN(TestWaylandOutput)
