@@ -18,7 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 // Qt
-#include <QtTest/QtTest>
+#include <QtTest>
 // KWin
 #include "../../src/client/compositor.h"
 #include "../../src/client/connection_thread.h"
@@ -55,6 +55,7 @@ private Q_SLOTS:
     void testToplevel();
     void testTransient_data();
     void testTransient();
+    void testTransientPopup();
     void testPing();
     void testTitle();
     void testWindowClass();
@@ -63,6 +64,9 @@ private Q_SLOTS:
     void testMove();
     void testResize_data();
     void testResize();
+    void testDisconnect();
+    void testWhileDestroying();
+    void testClientDisconnecting();
 
 private:
     KWayland::Server::Display *m_display;
@@ -243,6 +247,8 @@ void TestWaylandShell::testCreateMultiple()
     QScopedPointer<ShellSurface> surface1(m_shell->createSurface(s1.data()));
     QVERIFY(!surface1.isNull());
     QVERIFY(surface1->isValid());
+    QVERIFY(!ShellSurface::get(nullptr));
+    QCOMPARE(ShellSurface::get(*(surface1.data())), surface1.data());
 
     QVERIFY(serverSurfaceSpy.wait());
     QCOMPARE(serverSurfaceSpy.count(), 1);
@@ -250,6 +256,7 @@ void TestWaylandShell::testCreateMultiple()
     QScopedPointer<ShellSurface> surface2(m_shell->createSurface(s2.data()));
     QVERIFY(!surface2.isNull());
     QVERIFY(surface2->isValid());
+    QCOMPARE(ShellSurface::get(*(surface2.data())), surface2.data());
 
     QVERIFY(serverSurfaceSpy.wait());
     QCOMPARE(serverSurfaceSpy.count(), 2);
@@ -258,6 +265,7 @@ void TestWaylandShell::testCreateMultiple()
     QScopedPointer<ShellSurface> surface3(m_shell->createSurface(s2.data()));
     QVERIFY(!surface3.isNull());
     QVERIFY(surface3->isValid());
+    QCOMPARE(ShellSurface::get(*(surface3.data())), surface3.data());
 
     QVERIFY(!serverSurfaceSpy.wait(100));
     QCOMPARE(serverSurfaceSpy.count(), 2);
@@ -280,14 +288,17 @@ void TestWaylandShell::testFullscreen()
     ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
     QVERIFY(serverSurface);
     QVERIFY(serverSurface->parentResource());
+    QCOMPARE(serverSurface->shell(), m_shellInterface);
 
     QSignalSpy fullscreenSpy(serverSurface, SIGNAL(fullscreenChanged(bool)));
     QVERIFY(fullscreenSpy.isValid());
 
+    QVERIFY(!serverSurface->isFullscreen());
     surface->setFullscreen();
     QVERIFY(fullscreenSpy.wait());
     QCOMPARE(fullscreenSpy.count(), 1);
     QVERIFY(fullscreenSpy.first().first().toBool());
+    QVERIFY(serverSurface->isFullscreen());
     serverSurface->requestSize(QSize(1024, 768));
 
     QVERIFY(sizeSpy.wait());
@@ -301,6 +312,7 @@ void TestWaylandShell::testFullscreen()
     QVERIFY(fullscreenSpy.wait());
     QCOMPARE(fullscreenSpy.count(), 1);
     QVERIFY(!fullscreenSpy.first().first().toBool());
+    QVERIFY(!serverSurface->isFullscreen());
 }
 
 void TestWaylandShell::testMaximize()
@@ -324,10 +336,12 @@ void TestWaylandShell::testMaximize()
     QSignalSpy maximizedSpy(serverSurface, SIGNAL(maximizedChanged(bool)));
     QVERIFY(maximizedSpy.isValid());
 
+    QVERIFY(!serverSurface->isMaximized());
     surface->setMaximized();
     QVERIFY(maximizedSpy.wait());
     QCOMPARE(maximizedSpy.count(), 1);
     QVERIFY(maximizedSpy.first().first().toBool());
+    QVERIFY(serverSurface->isMaximized());
     serverSurface->requestSize(QSize(1024, 768));
 
     QVERIFY(sizeSpy.wait());
@@ -341,6 +355,7 @@ void TestWaylandShell::testMaximize()
     QVERIFY(maximizedSpy.wait());
     QCOMPARE(maximizedSpy.count(), 1);
     QVERIFY(!maximizedSpy.first().first().toBool());
+    QVERIFY(!serverSurface->isMaximized());
 }
 
 void TestWaylandShell::testToplevel()
@@ -461,6 +476,72 @@ void TestWaylandShell::testTransient()
     QVERIFY(serverSurface2->acceptsKeyboardFocus());
 }
 
+void TestWaylandShell::testTransientPopup()
+{
+    using namespace KWayland::Server;
+    using namespace KWayland::Client;
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    ShellSurface *surface = m_shell->createSurface(s.data(), m_shell);
+
+    QSignalSpy serverSurfaceSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(serverSurfaceSpy.isValid());
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface);
+    QCOMPARE(serverSurface->isToplevel(), true);
+    QCOMPARE(serverSurface->isPopup(), false);
+    QCOMPARE(serverSurface->isTransient(), false);
+    QCOMPARE(serverSurface->transientFor(), QPointer<SurfaceInterface>());
+    QCOMPARE(serverSurface->transientOffset(), QPoint());
+    QVERIFY(serverSurface->acceptsKeyboardFocus());
+
+    QSignalSpy transientSpy(serverSurface, &ShellSurfaceInterface::transientChanged);
+    QVERIFY(transientSpy.isValid());
+    QSignalSpy transientOffsetSpy(serverSurface, &ShellSurfaceInterface::transientOffsetChanged);
+    QVERIFY(transientOffsetSpy.isValid());
+    QSignalSpy transientForChangedSpy(serverSurface, &ShellSurfaceInterface::transientForChanged);
+    QVERIFY(transientForChangedSpy.isValid());
+
+    QScopedPointer<Surface> s2(m_compositor->createSurface());
+    m_shell->createSurface(s2.data(), m_shell);
+    serverSurfaceSpy.clear();
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface2 = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface2 != serverSurface);
+    QVERIFY(serverSurface2);
+    QVERIFY(serverSurface2->acceptsKeyboardFocus());
+
+    // TODO: proper serial checking
+    surface->setTransientPopup(s2.data(), m_seat, 1, QPoint(10, 20));
+    QVERIFY(transientSpy.wait());
+    QCOMPARE(transientSpy.count(), 1);
+    QCOMPARE(transientSpy.first().first().toBool(), true);
+    QCOMPARE(transientOffsetSpy.count(), 1);
+    QCOMPARE(transientOffsetSpy.first().first().toPoint(), QPoint(10, 20));
+    QCOMPARE(transientForChangedSpy.count(), 1);
+    QCOMPARE(serverSurface->isToplevel(), false);
+    QCOMPARE(serverSurface->isPopup(), true);
+    QCOMPARE(serverSurface->isTransient(), true);
+    QCOMPARE(serverSurface->transientFor(), QPointer<SurfaceInterface>(serverSurface2->surface()));
+    QCOMPARE(serverSurface->transientOffset(), QPoint(10, 20));
+    // TODO: honor the flag
+    QCOMPARE(serverSurface->acceptsKeyboardFocus(), false);
+
+    QCOMPARE(serverSurface2->isToplevel(), true);
+    QCOMPARE(serverSurface2->isPopup(), false);
+    QCOMPARE(serverSurface2->isTransient(), false);
+    QCOMPARE(serverSurface2->transientFor(), QPointer<SurfaceInterface>());
+    QCOMPARE(serverSurface2->transientOffset(), QPoint());
+
+    // send popup done
+    QSignalSpy popupDoneSpy(surface, &ShellSurface::popupDone);
+    QVERIFY(popupDoneSpy.isValid());
+    serverSurface->popupDone();
+    QVERIFY(popupDoneSpy.wait());
+}
+
 void TestWaylandShell::testPing()
 {
     using namespace KWayland::Server;
@@ -475,6 +556,7 @@ void TestWaylandShell::testPing()
     QVERIFY(serverSurfaceSpy.wait());
     ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
     QVERIFY(serverSurface);
+    QVERIFY(!serverSurface->isPinged());
 
     QSignalSpy pingTimeoutSpy(serverSurface, SIGNAL(pingTimeout()));
     QVERIFY(pingTimeoutSpy.isValid());
@@ -482,6 +564,7 @@ void TestWaylandShell::testPing()
     QVERIFY(pongSpy.isValid());
 
     serverSurface->ping();
+    QVERIFY(serverSurface->isPinged());
     QVERIFY(pingSpy.wait());
     wl_display_flush(m_connection->display());
 
@@ -523,7 +606,7 @@ void TestWaylandShell::testTitle()
     QString testTitle = QStringLiteral("fooBar");
     QVERIFY(serverSurface->title().isNull());
 
-    wl_shell_surface_set_title(*(const KWayland::Client::ShellSurface *)surface, testTitle.toUtf8().constData());
+    surface->setTitle(testTitle);
     QVERIFY(titleSpy.wait());
     QCOMPARE(serverSurface->title(), testTitle);
     QCOMPARE(titleSpy.first().first().toString(), testTitle);
@@ -548,10 +631,14 @@ void TestWaylandShell::testWindowClass()
     QByteArray testClass = QByteArrayLiteral("fooBar");
     QVERIFY(serverSurface->windowClass().isNull());
 
-    wl_shell_surface_set_class(*surface, testClass.constData());
+    surface->setWindowClass(testClass);
     QVERIFY(windowClassSpy.wait());
     QCOMPARE(serverSurface->windowClass(), testClass);
     QCOMPARE(windowClassSpy.first().first().toByteArray(), testClass);
+
+    // try setting it to same should not trigger the signal
+    surface->setWindowClass(testClass);
+    QVERIFY(!windowClassSpy.wait(100));
 }
 
 void TestWaylandShell::testDestroy()
@@ -701,5 +788,135 @@ void TestWaylandShell::testResize()
     QTEST(resizeRequestedSpy.first().at(2).value<Qt::Edges>(), "expectedEdge");
 }
 
+void TestWaylandShell::testDisconnect()
+{
+    // this test verifies that the server side correctly tears down the resources when the client disconnects
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    QScopedPointer<KWayland::Client::Surface> s(m_compositor->createSurface());
+    QVERIFY(!s.isNull());
+    QVERIFY(s->isValid());
+    QScopedPointer<ShellSurface> surface(m_shell->createSurface(s.data(), m_shell));
+    QSignalSpy serverSurfaceSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(serverSurfaceSpy.isValid());
+    QVERIFY(serverSurfaceSpy.wait());
+    ShellSurfaceInterface *serverSurface = serverSurfaceSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverSurface);
+
+    // destroy client
+    QSignalSpy clientDisconnectedSpy(serverSurface->client(), &ClientConnection::disconnected);
+    QVERIFY(clientDisconnectedSpy.isValid());
+    QSignalSpy shellSurfaceDestroyedSpy(serverSurface, &QObject::destroyed);
+    QVERIFY(shellSurfaceDestroyedSpy.isValid());
+    if (m_connection) {
+        m_connection->deleteLater();
+        m_connection = nullptr;
+    }
+    QVERIFY(clientDisconnectedSpy.wait());
+    QCOMPARE(clientDisconnectedSpy.count(), 1);
+    QCOMPARE(shellSurfaceDestroyedSpy.count(), 0);
+    // it's already unbound, but the shell surface is not yet destroyed
+    QVERIFY(!serverSurface->resource());
+    // ensure we don't crash when accessing it in this state
+    serverSurface->ping();
+    serverSurface->requestSize(QSize(1, 2));
+    QVERIFY(shellSurfaceDestroyedSpy.wait());
+    QCOMPARE(shellSurfaceDestroyedSpy.count(), 1);
+
+    s->destroy();
+    surface->destroy();
+    m_shell->destroy();
+    m_compositor->destroy();
+    m_pointer->destroy();
+    m_seat->destroy();
+    m_queue->destroy();
+}
+
+void TestWaylandShell::testWhileDestroying()
+{
+    // this test tries to hit a condition that a Surface gets created with an ID which was already
+    // used for a previous Surface. For each Surface we try to create a ShellSurface.
+    // Even if there was a Surface in the past with the same ID, it should create the ShellSurface
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    QSignalSpy surfaceCreatedSpy(m_compositorInterface, &CompositorInterface::surfaceCreated);
+    QVERIFY(surfaceCreatedSpy.isValid());
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QVERIFY(surfaceCreatedSpy.wait());
+    auto serverSurface = surfaceCreatedSpy.first().first().value<SurfaceInterface*>();
+    QVERIFY(serverSurface);
+
+    // create ShellSurface
+    QSignalSpy shellSurfaceCreatedSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(shellSurfaceCreatedSpy.isValid());
+    QScopedPointer<ShellSurface> ps(m_shell->createSurface(s.data()));
+    QVERIFY(shellSurfaceCreatedSpy.wait());
+
+    // now try to create more surfaces
+    QSignalSpy clientErrorSpy(m_connection, &ConnectionThread::errorOccurred);
+    QVERIFY(clientErrorSpy.isValid());
+    for (int i = 0; i < 100; i++) {
+        s.reset();
+        ps.reset();
+        s.reset(m_compositor->createSurface());
+        ps.reset(m_shell->createSurface(s.data()));
+        QVERIFY(surfaceCreatedSpy.wait());
+    }
+    QVERIFY(clientErrorSpy.isEmpty());
+    QVERIFY(!clientErrorSpy.wait(100));
+    QVERIFY(clientErrorSpy.isEmpty());
+}
+
+void TestWaylandShell::testClientDisconnecting()
+{
+    // this test tries to request a new surface size while the client is actually already destroyed
+    // see BUG: 370232
+    using namespace KWayland::Client;
+    using namespace KWayland::Server;
+    // create ShellSurface
+    QScopedPointer<Surface> s(m_compositor->createSurface());
+    QSignalSpy shellSurfaceCreatedSpy(m_shellInterface, &ShellInterface::surfaceCreated);
+    QVERIFY(shellSurfaceCreatedSpy.isValid());
+    QScopedPointer<ShellSurface> ps(m_shell->createSurface(s.data()));
+    QVERIFY(shellSurfaceCreatedSpy.wait());
+
+    auto serverShellSurface = shellSurfaceCreatedSpy.first().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverShellSurface);
+
+    QSignalSpy shellSurfaceUnboundSpy(serverShellSurface, &Resource::unbound);
+    QVERIFY(shellSurfaceUnboundSpy.isValid());
+
+    QScopedPointer<Surface> s2(m_compositor->createSurface());
+    QScopedPointer<ShellSurface> ps2(m_shell->createSurface(s2.data()));
+    QVERIFY(shellSurfaceCreatedSpy.wait());
+    auto serverShellSurface2 = shellSurfaceCreatedSpy.last().first().value<ShellSurfaceInterface*>();
+    QVERIFY(serverShellSurface2);
+
+    connect(serverShellSurface, &Resource::unbound, this,
+        [serverShellSurface, serverShellSurface2] {
+            serverShellSurface2->requestSize(QSize(100, 200));
+        }
+    );
+
+    m_connection->deleteLater();
+    m_connection = nullptr;
+    m_thread->quit();
+    m_thread->wait();
+    delete m_thread;
+    m_thread = nullptr;
+
+    QVERIFY(shellSurfaceUnboundSpy.wait());
+
+    ps->destroy();
+    s->destroy();
+    ps2->destroy();
+    s2->destroy();
+    m_pointer->destroy();
+    m_seat->destroy();
+    m_shell->destroy();
+    m_compositor->destroy();
+    m_queue->destroy();
+}
+
 QTEST_GUILESS_MAIN(TestWaylandShell)
 #include "test_wayland_shell.moc"
