@@ -18,7 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "connection_thread.h"
-#include "logging_p.h"
+#include "logging.h"
 // Qt
 #include <QAbstractEventDispatcher>
 #include <QGuiApplication>
@@ -197,6 +197,14 @@ ConnectionThread::ConnectionThread(QObject *parent)
         Qt::DirectConnection);
 }
 
+ConnectionThread::ConnectionThread(wl_display *display, QObject *parent)
+    : QObject(parent)
+    , d(new Private(this))
+{
+    d->display = display;
+    d->foreign = true;
+}
+
 ConnectionThread::~ConnectionThread()
 {
     disconnect(d->eventDispatcherConnection);
@@ -204,9 +212,6 @@ ConnectionThread::~ConnectionThread()
 
 ConnectionThread *ConnectionThread::fromApplication(QObject *parent)
 {
-    if (!QGuiApplication::platformName().contains(QStringLiteral("wayland"), Qt::CaseInsensitive)) {
-        return nullptr;
-    }
     QPlatformNativeInterface *native = qApp->platformNativeInterface();
     if (!native) {
         return nullptr;
@@ -215,9 +220,7 @@ ConnectionThread *ConnectionThread::fromApplication(QObject *parent)
     if (!display) {
         return nullptr;
     }
-    ConnectionThread *ct = new ConnectionThread(parent);
-    ct->d->foreign = true;
-    ct->d->display = display;
+    ConnectionThread *ct = new ConnectionThread(display, parent);
     connect(native, &QObject::destroyed, ct, &ConnectionThread::connectionDied);
     return ct;
 }
