@@ -36,7 +36,7 @@ class DataSource;
 class Surface;
 
 /**
- * @short Wrapper for the wl_data_device interface.
+ * @short DataDevice allows clients to share data by copy-and-paste and drag-and-drop.
  *
  * This class is a convenient wrapper for the wl_data_device interface.
  * To create a DataDevice call DataDeviceManager::getDataDevice.
@@ -71,10 +71,8 @@ public:
      * that the instance can be deleted or set up to a new wl_data_device interface
      * once there is a new connection available.
      *
-     * It is suggested to connect this method to ConnectionThread::connectionDied:
-     * @code
-     * connect(connection, &ConnectionThread::connectionDied, source, &DataDevice::destroy);
-     * @endcode
+     * This method is automatically invoked when the Registry which created this
+     * DataDevice gets destroyed.
      *
      * @see release
      **/
@@ -92,12 +90,62 @@ public:
 
     DataOffer *offeredSelection() const;
 
+    /**
+     * @returns the currently focused surface during drag'n'drop on this DataDevice.
+     * @since 5.22
+     **/
+    QPointer<Surface> dragSurface() const;
+    /**
+     * @returns the DataOffer during a drag'n'drop operation.
+     * @since 5.22
+     **/
+    DataOffer *dragOffer() const;
+
     operator wl_data_device*();
     operator wl_data_device*() const;
 
 Q_SIGNALS:
     void selectionOffered(KWayland::Client::DataOffer*);
     void selectionCleared();
+    /**
+     * Notification that a drag'n'drop operation entered a Surface on this DataDevice.
+     *
+     * @param serial The serial for this enter
+     * @param relativeToSurface Coordinates relative to the upper-left corner of the Surface.
+     * @see dragSurface
+     * @see dragOffer
+     * @see dragLeft
+     * @see dragMotion
+     * @since 5.22
+     **/
+    void dragEntered(quint32 serial, const QPointF &relativeToSurface);
+    /**
+     * Notification that the drag'n'drop operation left the Surface on this DataDevice.
+     *
+     * The leave notification is sent before the enter notification for the new focus.
+     * @see dragEnter
+     * @since 5.22
+     **/
+    void dragLeft();
+    /**
+     * Notification of drag motion events on the current drag surface.
+     *
+     * @param relativeToSurface  Coordinates relative to the upper-left corner of the entered Surface.
+     * @param time timestamp with millisecond granularity
+     * @see dragEntered
+     * @since 5.22
+     **/
+    void dragMotion(const QPointF &relativeToSurface, quint32 time);
+    /**
+     * Emitted when the implicit grab is removed and the drag'n'drop operation ended on this
+     * DataDevice.
+     *
+     * The client can now start a data transfer on the DataOffer.
+     * @see dragEntered
+     * @see dragOffer
+     * @since 5.22
+     **/
+    void dropped();
 
 private:
     class Private;
