@@ -29,7 +29,7 @@ namespace KWayland
 namespace Client
 {
 
-class OutputManagement::Private
+class Q_DECL_HIDDEN OutputManagement::Private
 {
 public:
     Private() = default;
