@@ -44,7 +44,7 @@ class Surface;
  * The model resets when the PlasmaWindowManagement parent signals that its
  * interface is about to be destroyed.
  *
- * To use this class you can create an instance yourself, or preferrably use the
+ * To use this class you can create an instance yourself, or preferably use the
  * convenience method in PlasmaWindowManagement:
  * @code
  * PlasmaWindowModel *model = wm->createWindowModel();
@@ -57,7 +57,6 @@ class Surface;
 class KWAYLANDCLIENT_EXPORT PlasmaWindowModel : public QAbstractListModel
 {
     Q_OBJECT
-    Q_ENUMS(AdditionalRoles)
 
 public:
     enum AdditionalRoles {
@@ -71,24 +70,72 @@ public:
         IsMinimized,
         IsKeepAbove,
         IsKeepBelow,
+#if KWAYLANDCLIENT_ENABLE_DEPRECATED_SINCE(5, 53)
+        /**
+          @deprecated Since 5.53, use VirtualDesktops
+         */
         VirtualDesktop,
+#else
+        VirtualDesktop_DEPRECATED_DO_NOT_USE,
+#endif
         IsOnAllDesktops,
         IsDemandingAttention,
-        SkipTaskbar
+        SkipTaskbar,
+        /**
+        * @since 5.22
+        */
+        IsShadeable,
+        /**
+        * @since 5.22
+        */
+        IsShaded,
+        /**
+        * @since 5.22
+        */
+        IsMovable,
+        /**
+        * @since 5.22
+        */
+        IsResizable,
+        /**
+        * @since 5.22
+        */
+        IsVirtualDesktopChangeable,
+        /**
+        * @since 5.22
+        */
+        IsCloseable,
+        /**
+        * @since 5.25
+        */
+        Geometry,
+        /**
+         * @since 5.35
+         */
+        Pid,
+        /**
+         * @since 5.47
+         */
+        SkipSwitcher,
+        /**
+         * @since 5.53
+         */
+        VirtualDesktops
     };
+    Q_ENUM(AdditionalRoles)
 
     explicit PlasmaWindowModel(PlasmaWindowManagement *parent);
     virtual ~PlasmaWindowModel();
 
-    virtual QHash<int, QByteArray> roleNames() const;
+    QHash<int, QByteArray> roleNames() const override;
 
-    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
-    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
 
     /**
      * Returns an index with internalPointer() pointing to a PlasmaWindow instance.
      **/
-    virtual QModelIndex	index(int row, int column = 0, const QModelIndex &parent = QModelIndex()) const;
+    QModelIndex	index(int row, int column = 0, const QModelIndex &parent = QModelIndex()) const override;
 
 
     /**
@@ -101,11 +148,35 @@ public:
      **/
     Q_INVOKABLE void requestClose(int row);
 
+    /**
+     * Request an interactive move for the window at this model row index.
+     * @since 5.22
+     **/
+    Q_INVOKABLE void requestMove(int row);
+
+    /**
+     * Request an interactive resize for the window at this model row index.
+     * @since 5.22
+     **/
+    Q_INVOKABLE void requestResize(int row);
+
     /**
      * Request the window at this model row index be moved to this virtual desktop.
      **/
     Q_INVOKABLE void requestVirtualDesktop(int row, quint32 desktop);
 
+    /**
+     * Requests the window at this model row index have its keep above state toggled.
+     * @since 5.35
+     */
+    Q_INVOKABLE void requestToggleKeepAbove(int row);
+
+    /**
+     * Requests the window at this model row index have its keep above state toggled.
+     * @since 5.35
+     */
+    Q_INVOKABLE void requestToggleKeepBelow(int row);
+
     /**
      * Requests the window at this model row index have its minimized state toggled.
      */
@@ -123,6 +194,11 @@ public:
      */
     Q_INVOKABLE void setMinimizedGeometry(int row, Surface *panel, const QRect &geom);
 
+    /**
+     * Requests the window at this model row index have its shaded state toggled.
+     * @since 5.22
+     */
+    Q_INVOKABLE void requestToggleShaded(int row);
 
 private:
     class Private;
