@@ -28,6 +28,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "fullscreen_shell.h"
 #include "idle.h"
 #include "idleinhibit.h"
+#include "keystate.h"
 #include "remote_access.h"
 #include "logging.h"
 #include "outputconfiguration.h"
@@ -91,6 +92,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include <wayland-server-decoration-palette-client-protocol.h>
 #include <wayland-xdg-output-unstable-v1-client-protocol.h>
 #include <wayland-xdg-decoration-unstable-v1-client-protocol.h>
+#include <wayland-keystate-client-protocol.h>
 
 /*****
  * How to add another interface:
@@ -120,7 +122,7 @@ struct SuppertedInterfaceData {
 };
 static const QMap<Registry::Interface, SuppertedInterfaceData> s_interfaces = {
     {Registry::Interface::Compositor, {
-        3,
+        4,
         QByteArrayLiteral("wl_compositor"),
         &wl_compositor_interface,
         &Registry::compositorAnnounced,
@@ -169,21 +171,21 @@ static const QMap<Registry::Interface, SuppertedInterfaceData> s_interfaces = {
         &Registry::subCompositorRemoved
     }},
     {Registry::Interface::PlasmaShell, {
-        5,
+        6,
         QByteArrayLiteral("org_kde_plasma_shell"),
         &org_kde_plasma_shell_interface,
         &Registry::plasmaShellAnnounced,
         &Registry::plasmaShellRemoved
     }},
     {Registry::Interface::PlasmaVirtualDesktopManagement, {
-        1,
+        2,
         QByteArrayLiteral("org_kde_plasma_virtual_desktop_management"),
         &org_kde_plasma_virtual_desktop_management_interface,
         &Registry::plasmaVirtualDesktopManagementAnnounced,
         &Registry::plasmaVirtualDesktopManagementRemoved
     }},
     {Registry::Interface::PlasmaWindowManagement, {
-        9,
+        10,
         QByteArrayLiteral("org_kde_plasma_window_management"),
         &org_kde_plasma_window_management_interface,
         &Registry::plasmaWindowManagementAnnounced,
@@ -204,7 +206,7 @@ static const QMap<Registry::Interface, SuppertedInterfaceData> s_interfaces = {
         &Registry::remoteAccessManagerRemoved
     }},
     {Registry::Interface::FakeInput, {
-        2,
+        4,
         QByteArrayLiteral("org_kde_kwin_fake_input"),
         &org_kde_kwin_fake_input_interface,
         &Registry::fakeInputAnnounced,
@@ -377,6 +379,13 @@ static const QMap<Registry::Interface, SuppertedInterfaceData> s_interfaces = {
         &zxdg_decoration_manager_v1_interface,
         &Registry::xdgDecorationAnnounced,
         &Registry::xdgDecorationRemoved
+    }},
+    {Registry::Interface::Keystate, {
+        1,
+        QByteArrayLiteral("org_kde_kwin_keystate"),
+        &org_kde_kwin_keystate_interface,
+        &Registry::keystateAnnounced,
+        &Registry::keystateRemoved
     }}
 };
 
@@ -505,7 +514,7 @@ EventQueue *Registry::eventQueue()
     return d->queue;
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct wl_registry_listener Registry::Private::s_registryListener = {
     globalAnnounce,
     globalRemove
@@ -536,7 +545,7 @@ void Registry::Private::globalSync(void* data, wl_callback* callback, uint32_t s
     auto r = reinterpret_cast<Registry::Private*>(data);
     Q_ASSERT(r->callback == callback);
     r->handleGlobalSync();
-    r->callback.destroy();
+    r->callback.release();
 }
 
 void Registry::Private::handleGlobalSync()
@@ -547,7 +556,7 @@ void Registry::Private::handleGlobalSync()
 namespace {
 static Registry::Interface nameToInterface(const char *interface)
 {
-    for (auto it = s_interfaces.begin(); it != s_interfaces.end(); ++it) {
+    for (auto it = s_interfaces.constBegin(); it != s_interfaces.constEnd(); ++it) {
         if (qstrcmp(interface, it.value().name) == 0) {
             return it.key();
         }
@@ -592,12 +601,12 @@ void Registry::Private::handleRemove(uint32_t name)
 
 bool Registry::Private::hasInterface(Registry::Interface interface) const
 {
-    auto it = std::find_if(m_interfaces.begin(), m_interfaces.end(),
+    auto it = std::find_if(m_interfaces.constBegin(), m_interfaces.constEnd(),
         [interface](const InterfaceData &data) {
             return data.interface == interface;
         }
     );
-    return it != m_interfaces.end();
+    return it != m_interfaces.constEnd();
 }
 
 QVector<Registry::AnnouncedInterface> Registry::Private::interfaces(Interface interface) const
@@ -684,6 +693,7 @@ BIND(PointerConstraintsUnstableV1, zwp_pointer_constraints_v1)
 BIND(XdgExporterUnstableV2, zxdg_exporter_v2)
 BIND(XdgImporterUnstableV2, zxdg_importer_v2)
 BIND(IdleInhibitManagerUnstableV1, zwp_idle_inhibit_manager_v1)
+BIND(Keystate, org_kde_kwin_keystate)
 BIND2(ShadowManager, Shadow, org_kde_kwin_shadow_manager)
 BIND2(BlurManager, Blur, org_kde_kwin_blur_manager)
 BIND2(ContrastManager, Contrast, org_kde_kwin_contrast_manager)
@@ -745,6 +755,7 @@ CREATE(DpmsManager)
 CREATE(ServerSideDecorationManager)
 CREATE2(ShmPool, Shm)
 CREATE(AppMenuManager)
+CREATE(Keystate)
 CREATE(ServerSideDecorationPaletteManager)
 
 #undef CREATE
@@ -862,10 +873,10 @@ static const wl_interface *wlInterface(Registry::Interface interface)
 template <typename T>
 T *Registry::Private::bind(Registry::Interface interface, uint32_t name, uint32_t version) const
 {
-    auto it = std::find_if(m_interfaces.begin(), m_interfaces.end(), [=](const InterfaceData &data) {
+    auto it = std::find_if(m_interfaces.constBegin(), m_interfaces.constEnd(), [=](const InterfaceData &data) {
         return data.interface == interface && data.name == name && data.version >= version;
     });
-    if (it == m_interfaces.end()) {
+    if (it == m_interfaces.constEnd()) {
         qCDebug(KWAYLAND_CLIENT) << "Don't have interface " << int(interface) << "with name " << name << "and minimum version" << version;
         return nullptr;
     }
