@@ -1,23 +1,38 @@
-/*
-    SPDX-FileCopyrightText: 2015 Martin Gräßlin <mgraesslin@kde.org>
-    SPDX-FileCopyrightText: 2015 Marco Martin <mart@kde.org>
+/********************************************************************
+Copyright 2015  Martin Gräßlin <mgraesslin@kde.org>
+Copyright 2015  Marco Martin <mart@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
-*/
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) version 3, or any
+later version accepted by the membership of KDE e.V. (or its
+successor approved by the membership of KDE e.V.), which shall
+act as a proxy defined in Section 6 of version 3 of the license.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+*********************************************************************/
 #include "blur_interface.h"
+#include "region_interface.h"
 #include "display.h"
 #include "global_p.h"
-#include "region_interface.h"
 #include "resource_p.h"
 #include "surface_interface_p.h"
 
-#include <wayland-blur-server-protocol.h>
 #include <wayland-server.h>
+#include <wayland-blur-server-protocol.h>
 
 namespace KWayland
 {
 namespace Server
 {
+
 class BlurManagerInterface::Private : public Global::Private
 {
 public:
@@ -30,11 +45,10 @@ private:
     static void createCallback(wl_client *client, wl_resource *resource, uint32_t id, wl_resource *surface);
     static void unsetCallback(wl_client *client, wl_resource *resource, wl_resource *surface);
     static void unbind(wl_resource *resource);
-    static Private *cast(wl_resource *r)
-    {
-        auto blurManager = reinterpret_cast<QPointer<BlurManagerInterface> *>(wl_resource_get_user_data(r))->data();
+    static Private *cast(wl_resource *r) {
+        auto blurManager = reinterpret_cast<QPointer<BlurManagerInterface>*>(wl_resource_get_user_data(r))->data();
         if (blurManager) {
-            return static_cast<Private *>(blurManager->d.data());
+            return static_cast<Private*>(blurManager->d.data());
         }
         return nullptr;
     }
@@ -46,7 +60,10 @@ private:
 const quint32 BlurManagerInterface::Private::s_version = 1;
 
 #ifndef K_DOXYGEN
-const struct org_kde_kwin_blur_manager_interface BlurManagerInterface::Private::s_interface = {createCallback, unsetCallback};
+const struct org_kde_kwin_blur_manager_interface BlurManagerInterface::Private::s_interface = {
+    createCallback,
+    unsetCallback
+};
 #endif
 
 BlurManagerInterface::Private::Private(BlurManagerInterface *q, Display *d)
@@ -63,20 +80,20 @@ void BlurManagerInterface::Private::bind(wl_client *client, uint32_t version, ui
         wl_client_post_no_memory(client);
         return;
     }
-    auto ref = new QPointer<BlurManagerInterface>(q); // deleted in unbind
+    auto ref = new QPointer<BlurManagerInterface>(q);//deleted in unbind
     wl_resource_set_implementation(resource, &s_interface, ref, unbind);
 }
 
 void BlurManagerInterface::Private::unbind(wl_resource *r)
 {
-    delete reinterpret_cast<QPointer<BlurManagerInterface> *>(wl_resource_get_user_data(r));
+    delete reinterpret_cast<QPointer<BlurManagerInterface>*>(wl_resource_get_user_data(r));
 }
 
 void BlurManagerInterface::Private::createCallback(wl_client *client, wl_resource *resource, uint32_t id, wl_resource *surface)
 {
     auto m = cast(resource);
     if (!m) {
-        return; // will happen if global is deleted
+        return;// will happen if global is deleted
     }
     m->createBlur(client, resource, id, surface);
 }
@@ -120,16 +137,15 @@ class BlurInterface::Private : public Resource::Private
 {
 public:
     Private(BlurInterface *q, BlurManagerInterface *c, wl_resource *parentResource);
-    ~Private() override;
+    ~Private();
 
     QRegion pendingRegion;
     QRegion currentRegion;
 
 private:
     void commit();
-    // TODO
-    BlurInterface *q_func()
-    {
+    //TODO
+    BlurInterface *q_func() {
         return reinterpret_cast<BlurInterface *>(q);
     }
 
@@ -140,7 +156,11 @@ private:
 };
 
 #ifndef K_DOXYGEN
-const struct org_kde_kwin_blur_interface BlurInterface::Private::s_interface = {commitCallback, setRegionCallback, resourceDestroyedCallback};
+const struct org_kde_kwin_blur_interface BlurInterface::Private::s_interface = {
+    commitCallback,
+    setRegionCallback,
+    resourceDestroyedCallback
+};
 #endif
 
 void BlurInterface::Private::commitCallback(wl_client *client, wl_resource *resource)
@@ -188,10 +208,8 @@ QRegion BlurInterface::region()
 
 BlurInterface::Private *BlurInterface::d_func() const
 {
-    return reinterpret_cast<Private *>(d.data());
+    return reinterpret_cast<Private*>(d.data());
 }
 
 }
 }
-
-#include "moc_blur_interface.cpp"
