@@ -51,7 +51,7 @@ private:
     static const quint32 s_version;
 };
 
-const quint32 CompositorInterface::Private::s_version = 3;
+const quint32 CompositorInterface::Private::s_version = 4;
 
 CompositorInterface::Private::Private(CompositorInterface *q, Display *d)
     : Global::Private(d, &wl_compositor_interface, s_version)
@@ -59,7 +59,7 @@ CompositorInterface::Private::Private(CompositorInterface *q, Display *d)
 {
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct wl_compositor_interface CompositorInterface::Private::s_interface = {
     createSurfaceCallback,
     createRegionCallback
@@ -105,14 +105,6 @@ void CompositorInterface::Private::createSurface(wl_client *client, wl_resource
         delete surface;
         return;
     }
-    QObject::connect(surface->client(), &ClientConnection::disconnected, surface,
-        [surface] {
-            if (surface->resource()) {
-                wl_resource_destroy(surface->resource());
-                delete surface;
-            }
-        }
-    );
     emit q->surfaceCreated(surface);
 }
 
@@ -130,14 +122,6 @@ void CompositorInterface::Private::createRegion(wl_client *client, wl_resource *
         delete region;
         return;
     }
-    QObject::connect(region->client(), &ClientConnection::disconnected, region,
-        [region] {
-            if (region->resource()) {
-                wl_resource_destroy(region->resource());
-                delete region;
-            }
-        }
-    );
     emit q->regionCreated(region);
 }
 
