@@ -28,15 +28,15 @@ namespace Server
 
 const quint32 DpmsManagerInterface::Private::s_version = 1;
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct org_kde_kwin_dpms_manager_interface DpmsManagerInterface::Private::s_interface = {
     getDpmsCallback
 };
 #endif
 
-DpmsManagerInterface::Private::Private(DpmsManagerInterface *q, Display *d)
+DpmsManagerInterface::Private::Private(DpmsManagerInterface *qptr, Display *d)
     : Global::Private(d, &org_kde_kwin_dpms_manager_interface, s_version)
-    , q(q)
+    , q(qptr)
 {
 }
 
@@ -75,16 +75,16 @@ DpmsManagerInterface::DpmsManagerInterface(Display *display, QObject *parent)
 DpmsManagerInterface::~DpmsManagerInterface() = default;
 
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct org_kde_kwin_dpms_interface DpmsInterface::Private::s_interface = {
     setCallback,
-    releaseCallback
+    resourceDestroyedCallback
 };
 #endif
 
-DpmsInterface::Private::Private(DpmsInterface *q, DpmsManagerInterface *g, wl_resource *parentResource, OutputInterface *output)
+DpmsInterface::Private::Private(DpmsInterface *q, DpmsManagerInterface *g, wl_resource *parentResource, OutputInterface *outputInterface)
     : Resource::Private(q, g, parentResource, &org_kde_kwin_dpms_interface, &s_interface)
-    , output(output)
+    , output(outputInterface)
 {
 }
 
@@ -111,16 +111,8 @@ void DpmsInterface::Private::setCallback(wl_client *client, wl_resource *resourc
     emit cast<Private>(resource)->output->dpmsModeRequested(dpmsMode);
 }
 
-void DpmsInterface::Private::releaseCallback(wl_client *client, wl_resource *resource)
-{
-    Q_UNUSED(client)
-    Private *p = reinterpret_cast<Private*>(wl_resource_get_user_data(resource));
-    wl_resource_destroy(resource);
-    p->q->deleteLater();
-}
-
 DpmsInterface::DpmsInterface(OutputInterface *output, wl_resource *parentResource, DpmsManagerInterface *manager)
-    : Resource(new Private(this, manager, parentResource, output), manager)
+    : Resource(new Private(this, manager, parentResource, output))
 {
     connect(output, &OutputInterface::dpmsSupportedChanged, this,
         [this] {
