@@ -62,6 +62,7 @@ public:
 
 private:
     static Private *cast(wl_resource *native);
+    static void releaseCallback(wl_client *client, wl_resource *resource);
     static void unbind(wl_resource *resource);
     void bind(wl_client *client, uint32_t version, uint32_t id) override;
     int32_t toTransform() const;
@@ -71,11 +72,12 @@ private:
 
     OutputInterface *q;
     static QVector<Private*> s_privates;
+    static const struct wl_output_interface s_interface;
     static const quint32 s_version;
 };
 
 QVector<OutputInterface::Private*> OutputInterface::Private::s_privates;
-const quint32 OutputInterface::Private::s_version = 2;
+const quint32 OutputInterface::Private::s_version = 3;
 
 OutputInterface::Private::Private(OutputInterface *q, Display *d)
     : Global::Private(d, &wl_output_interface, s_version)
@@ -89,6 +91,18 @@ OutputInterface::Private::~Private()
     s_privates.removeAll(this);
 }
 
+#ifndef K_DOXYGEN
+const struct wl_output_interface OutputInterface::Private::s_interface = {
+    releaseCallback
+};
+#endif
+
+void OutputInterface::Private::releaseCallback(wl_client *client, wl_resource *resource)
+{
+    Q_UNUSED(client);
+    unbind(resource);
+}
+
 OutputInterface *OutputInterface::Private::get(wl_resource *native)
 {
     if (Private *p = cast(native)) {
@@ -101,8 +115,8 @@ OutputInterface::Private *OutputInterface::Private::cast(wl_resource *native)
 {
     for (auto it = s_privates.constBegin(); it != s_privates.constEnd(); ++it) {
         const auto &resources = (*it)->resources;
-        auto rit = std::find_if(resources.begin(), resources.end(), [native] (const ResourceData &data) { return data.resource == native; });
-        if (rit != resources.end()) {
+        auto rit = std::find_if(resources.constBegin(), resources.constEnd(), [native] (const ResourceData &data) { return data.resource == native; });
+        if (rit != resources.constEnd()) {
             return (*it);
         }
     }
@@ -139,12 +153,12 @@ OutputInterface::~OutputInterface() = default;
 QSize OutputInterface::pixelSize() const
 {
     Q_D();
-    auto it = std::find_if(d->modes.begin(), d->modes.end(),
+    auto it = std::find_if(d->modes.constBegin(), d->modes.constEnd(),
         [](const Mode &mode) {
             return mode.flags.testFlag(ModeFlag::Current);
         }
     );
-    if (it == d->modes.end()) {
+    if (it == d->modes.constEnd()) {
         return QSize();
     }
     return (*it).size;
@@ -153,12 +167,12 @@ QSize OutputInterface::pixelSize() const
 int OutputInterface::refreshRate() const
 {
     Q_D();
-    auto it = std::find_if(d->modes.begin(), d->modes.end(),
+    auto it = std::find_if(d->modes.constBegin(), d->modes.constEnd(),
         [](const Mode &mode) {
             return mode.flags.testFlag(ModeFlag::Current);
         }
     );
-    if (it == d->modes.end()) {
+    if (it == d->modes.constEnd()) {
         return 60000;
     }
     return (*it).refreshRate;
@@ -303,7 +317,7 @@ void OutputInterface::Private::bind(wl_client *client, uint32_t version, uint32_
         return;
     }
     wl_resource_set_user_data(resource, this);
-    wl_resource_set_destructor(resource, unbind);
+    wl_resource_set_implementation(resource, &s_interface, this, unbind);
     ResourceData r;
     r.resource = resource;
     r.version = version;
@@ -506,6 +520,18 @@ bool OutputInterface::isDpmsSupported() const
     return d->dpms.supported;
 }
 
+QVector<wl_resource *> OutputInterface::clientResources(ClientConnection *client) const
+{
+    Q_D();
+    QVector<wl_resource *> ret;
+    for (auto it = d->resources.constBegin(), end = d->resources.constEnd(); it != end; ++it) {
+        if (wl_resource_get_client((*it).resource) == client->client()) {
+            ret << (*it).resource;
+        }
+    }
+    return ret;
+}
+
 OutputInterface *OutputInterface::get(wl_resource* native)
 {
     return Private::get(native);
