@@ -20,7 +20,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "outputconfiguration_interface.h"
 #include "outputdevice_interface.h"
-#include "logging_p.h"
+#include "logging.h"
 #include "resource_p.h"
 #include "display.h"
 #include "outputchangeset_p.h"
@@ -287,6 +287,9 @@ void OutputConfigurationInterface::setApplied()
 
 void OutputConfigurationInterface::Private::sendApplied()
 {
+    if (!resource) {
+        return;
+    }
     org_kde_kwin_outputconfiguration_send_applied(resource);
 }
 
@@ -299,6 +302,9 @@ void OutputConfigurationInterface::setFailed()
 
 void OutputConfigurationInterface::Private::sendFailed()
 {
+    if (!resource) {
+        return;
+    }
     org_kde_kwin_outputconfiguration_send_failed(resource);
 }
 
