@@ -24,6 +24,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "surface_interface.h"
 
 #include <QTimer>
+#include <QVariant>
 
 #include <wayland-server.h>
 #include <wayland-qt-surface-extension-server-protocol.h>
@@ -58,7 +59,7 @@ QtSurfaceExtensionInterface::Private::Private(QtSurfaceExtensionInterface *q, Di
 {
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct qt_surface_extension_interface QtSurfaceExtensionInterface::Private::s_interface = {
     createSurfaceCallback
 };
@@ -119,7 +120,7 @@ void QtSurfaceExtensionInterface::Private::createSurface(wl_client *client, uint
         }
     );
     if (it != surfaces.constEnd()) {
-        wl_resource_post_error(surface->resource(), WL_DISPLAY_ERROR_INVALID_OBJECT, "PlasmaShellSurface already created");
+        wl_resource_post_error(surface->resource(), WL_DISPLAY_ERROR_INVALID_OBJECT, "Qt Surface Extension already created");
         return;
     }
     QtExtendedSurfaceInterface *shellSurface = new QtExtendedSurfaceInterface(q, surface, parentResource);
@@ -142,7 +143,7 @@ QtExtendedSurfaceInterface::Private::Private(QtExtendedSurfaceInterface *q, QtSu
 {
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct qt_extended_surface_interface QtExtendedSurfaceInterface::Private::s_interface = {
     updateGenericPropertyCallback,
     setContentOrientationMaskCallback,
@@ -181,14 +182,23 @@ void QtExtendedSurfaceInterface::Private::setWindowFlagsCallback(wl_client *clie
 void QtExtendedSurfaceInterface::Private::updateGenericPropertyCallback(wl_client *client, wl_resource *resource, const char *name, wl_array *value)
 {
     Q_UNUSED(client)
-    Q_UNUSED(resource)
-    Q_UNUSED(name)
-    Q_UNUSED(value)
+    QByteArray data = QByteArray::fromRawData(static_cast<char *>(value->data), value->size);
+
+    QVariant variantValue;
+    QDataStream ds(data);
+    ds >> variantValue;
+    cast<Private>(resource)->q_func()->setProperty(name, variantValue);
 }
 
 QtExtendedSurfaceInterface::QtExtendedSurfaceInterface(QtSurfaceExtensionInterface *shell, SurfaceInterface *parent, wl_resource *parentResource)
-    : Resource(new Private(this, shell, parent, parentResource), parent)
+    : Resource(new Private(this, shell, parent, parentResource))
 {
+    auto unsetSurface = [this] {
+        Q_D();
+        d->surface = nullptr;
+    };
+    connect(parent, &Resource::unbound, this, unsetSurface);
+    connect(parent, &QObject::destroyed, this, unsetSurface);
 }
 
 QtExtendedSurfaceInterface::~QtExtendedSurfaceInterface() = default;
@@ -213,6 +223,9 @@ QtExtendedSurfaceInterface::Private *QtExtendedSurfaceInterface::d_func() const
 void QtExtendedSurfaceInterface::close()
 {
     Q_D();
+    if (!d->resource) {
+        return;
+    }
     qt_extended_surface_send_close(d->resource);
     d->client->flush();
 }
