@@ -57,7 +57,7 @@ SeatInterface::Private::Private(SeatInterface *q, Display *display)
 {
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct wl_seat_interface SeatInterface::Private::s_interface = {
     getPointerCallback,
     getKeyboardCallback,
@@ -71,13 +71,13 @@ SeatInterface::SeatInterface(Display *display, QObject *parent)
 {
     Q_D();
     connect(this, &SeatInterface::nameChanged, this,
-        [this, d] {
+        [d] {
             for (auto it = d->resources.constBegin(); it != d->resources.constEnd(); ++it) {
                 d->sendName(*it);
             }
         }
     );
-    auto sendCapabilitiesAll = [this, d] {
+    auto sendCapabilitiesAll = [d] {
         for (auto it = d->resources.constBegin(); it != d->resources.constEnd(); ++it) {
             d->sendCapabilities(*it);
         }
@@ -192,7 +192,7 @@ T *interfaceForSurface(SurfaceInterface *surface, const QVector<T*> &interfaces)
         return nullptr;
     }
 
-    for (auto it = interfaces.begin(); it != interfaces.end(); ++it) {
+    for (auto it = interfaces.constBegin(); it != interfaces.constEnd(); ++it) {
         if ((*it)->client() == surface->client()) {
             return (*it);
         }
@@ -209,7 +209,7 @@ QVector<T *> interfacesForSurface(SurfaceInterface *surface, const QVector<T*> &
         return ret;
     }
 
-    for (auto it = interfaces.begin(); it != interfaces.end(); ++it) {
+    for (auto it = interfaces.constBegin(); it != interfaces.constEnd(); ++it) {
         if ((*it)->client() == surface->client() && (*it)->resource()) {
             ret << *it;
         }
@@ -225,7 +225,7 @@ bool forEachInterface(SurfaceInterface *surface, const QVector<T*> &interfaces,
         return false;
     }
     bool calledAtLeastOne = false;
-    for (auto it = interfaces.begin(); it != interfaces.end(); ++it) {
+    for (auto it = interfaces.constBegin(); it != interfaces.constEnd(); ++it) {
         if ((*it)->client() == surface->client() && (*it)->resource()) {
             method(*it);
             calledAtLeastOne = true;
@@ -273,6 +273,7 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
         if (currentSelection == dataDevice) {
             // current selection is cleared
             currentSelection = nullptr;
+            emit q->selectionChanged(nullptr);
             if (keys.focus.selection) {
                 keys.focus.selection->sendClearSelection();
             }
@@ -306,9 +307,17 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
                 // no implicit grab, abort drag
                 return;
             }
+            auto *originSurface = dataDevice->origin();
+            const bool proxied = originSurface->dataProxy();
+            if (!proxied) {
+                // origin surface
+                drag.target = dataDevice;
+                drag.surface = originSurface;
+                // TODO: transformation needs to be either pointer or touch
+                drag.transformation = globalPointer.focus.transformation;
+            }
             drag.source = dataDevice;
-            drag.target = dataDevice;
-            drag.surface = dragSurface;
+            drag.sourcePointer = interfaceForSurface(originSurface, pointers);
             drag.destroyConnection = QObject::connect(dataDevice, &QObject::destroyed, q,
                 [this] {
                     endDrag(display->nextSerial());
@@ -328,7 +337,7 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
             } else {
                 drag.dragSourceDestroyConnection = QMetaObject::Connection();
             }
-            dataDevice->updateDragTarget(dataDevice->origin(), dataDevice->dragImplicitGrabSerial());
+            dataDevice->updateDragTarget(proxied ? nullptr : originSurface, dataDevice->dragImplicitGrabSerial());
             emit q->dragStarted();
             emit q->dragSurfaceChanged();
         }
@@ -405,6 +414,7 @@ void SeatInterface::Private::cancelPreviousSelection(DataDeviceInterface *dataDe
 
 void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice, bool set)
 {
+    bool selChanged = currentSelection != dataDevice;
     if (keys.focus.surface && (keys.focus.surface->client() == dataDevice->client())) {
         // cancel the previous selection
         cancelPreviousSelection(dataDevice);
@@ -419,9 +429,13 @@ void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice, bo
             } else {
                 keys.focus.selection->sendClearSelection();
                 currentSelection = nullptr;
+                selChanged = true;
             }
         }
     }
+    if (selChanged) {
+        emit q->selectionChanged(currentSelection);
+    }
 }
 
 void SeatInterface::setHasKeyboard(bool has)
@@ -832,6 +846,20 @@ bool SeatInterface::isPointerButtonPressed(quint32 button) const
     return it.value() == Private::Pointer::State::Pressed ? true : false;
 }
 
+void SeatInterface::pointerAxisV5(Qt::Orientation orientation, qreal delta, qint32 discreteDelta, PointerAxisSource source)
+{
+    Q_D();
+    if (d->drag.mode == Private::Drag::Mode::Pointer) {
+        // ignore
+        return;
+    }
+    if (d->globalPointer.focus.surface) {
+        for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
+            (*it)->axis(orientation, delta, discreteDelta, source);
+        }
+    }
+}
+
 void SeatInterface::pointerAxis(Qt::Orientation orientation, quint32 delta)
 {
     Q_D();
@@ -865,11 +893,11 @@ void SeatInterface::pointerButtonPressed(quint32 button)
         // ignore
         return;
     }
-    if (d->globalPointer.focus.surface) {
+    if (auto *focusSurface = d->globalPointer.focus.surface) {
         for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
             (*it)->buttonPressed(button, serial);
         }
-        if (d->globalPointer.focus.surface == d->keys.focus.surface) {
+        if (focusSurface == d->keys.focus.surface) {
             // update the focused child surface
             auto p = focusedPointer();
             if (p) {
@@ -1243,7 +1271,7 @@ QVector< quint32 > SeatInterface::pressedKeys() const
 {
     Q_D();
     QVector<quint32> keys;
-    for (auto it = d->keys.states.begin(); it != d->keys.states.end(); ++it) {
+    for (auto it = d->keys.states.constBegin(); it != d->keys.states.constEnd(); ++it) {
         if (it.value() == Private::Keyboard::State::Pressed) {
             keys << it.key();
         }
@@ -1346,7 +1374,7 @@ void SeatInterface::setFocusedTouchSurfacePosition(const QPointF &surfacePositio
 qint32 SeatInterface::touchDown(const QPointF &globalPosition)
 {
     Q_D();
-    const qint32 id = d->globalTouch.ids.isEmpty() ? 0 : d->globalTouch.ids.last() + 1;
+    const qint32 id = d->globalTouch.ids.isEmpty() ? 0 : d->globalTouch.ids.lastKey() + 1;
     const qint32 serial = display()->nextSerial();
     const auto pos = globalPosition - d->globalTouch.focus.offset;
     for (auto it = d->globalTouch.focus.touchs.constBegin(), end = d->globalTouch.focus.touchs.constEnd(); it != end; ++it) {
@@ -1474,7 +1502,7 @@ bool SeatInterface::hasImplicitPointerGrab(quint32 serial) const
 {
     Q_D();
     const auto &serials = d->globalPointer.buttonSerials;
-    for (auto it = serials.begin(), end = serials.end(); it != end; it++) {
+    for (auto it = serials.constBegin(), end = serials.constEnd(); it != end; it++) {
         if (it.value() == serial) {
             return isPointerButtonPressed(it.key());
         }
@@ -1580,6 +1608,7 @@ void SeatInterface::setSelection(DataDeviceInterface *dataDevice)
             d->keys.focus.selection->sendClearSelection();
         }
     }
+    emit selectionChanged(dataDevice);
 }
 
 }
