@@ -18,6 +18,7 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "shell_interface.h"
+#include "generic_shell_surface_p.h"
 #include "global_p.h"
 #include "resource_p.h"
 #include "display.h"
@@ -57,22 +58,21 @@ ShellInterface::Private::Private(ShellInterface *q, Display *d)
 {
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct wl_shell_interface ShellInterface::Private::s_interface = {
     createSurfaceCallback
 };
 #endif
 
 
-class ShellSurfaceInterface::Private : public Resource::Private
+class ShellSurfaceInterface::Private : public Resource::Private, public GenericShellSurface<ShellSurfaceInterface>
 {
 public:
     Private(ShellSurfaceInterface *q, ShellInterface *shell, SurfaceInterface *surface, wl_resource *parentResource);
     void ping();
 
-    SurfaceInterface *surface;
-    QString title;
-    QByteArray windowClass;
+    void commit() override;
+
     QScopedPointer<QTimer> pingTimer;
     quint32 pingSerial = 0;
     enum class WindowMode {
@@ -87,12 +87,13 @@ public:
     bool acceptsKeyboardFocus = true;
     void setWindowMode(WindowMode newWindowMode);
 
+    ShellSurfaceInterface *q_func() {
+        return reinterpret_cast<ShellSurfaceInterface *>(q);
+    }
+
 private:
     // interface callbacks
     static void pongCallback(wl_client *client, wl_resource *resource, uint32_t serial);
-    static void moveCallback(wl_client *client, wl_resource *resource, wl_resource *seat, uint32_t serial);
-    static void resizeCallback(wl_client *client, wl_resource *resource, wl_resource *seat,
-                               uint32_t serial, uint32_t edges);
     static void setToplevelCallback(wl_client *client, wl_resource *resource);
     static void setTransientCallback(wl_client *client, wl_resource *resource, wl_resource *parent,
                                      int32_t x, int32_t y, uint32_t flags);
@@ -101,16 +102,9 @@ private:
     static void setPopupCallback(wl_client *client, wl_resource *resource, wl_resource *seat, uint32_t serial,
                                 wl_resource *parent, int32_t x, int32_t y, uint32_t flags);
     static void setMaximizedCallback(wl_client *client, wl_resource *resource, wl_resource *output);
-    static void setTitleCallback(wl_client *client, wl_resource *resource, const char *title);
-    static void setClassCallback(wl_client *client, wl_resource *resource, const char *class_);
 
-    void setTitle(const QString &title);
-    void setWindowClass(const QByteArray &windowClass);
     void pong(quint32 serial);
     void setAcceptsFocus(quint32 flags);
-    ShellSurfaceInterface *q_func() {
-        return reinterpret_cast<ShellSurfaceInterface *>(q);
-    }
 
     static const struct wl_shell_surface_interface s_interface;
 };
@@ -166,25 +160,25 @@ void ShellInterface::Private::createSurface(wl_client *client, uint32_t version,
  *********************************/
 ShellSurfaceInterface::Private::Private(ShellSurfaceInterface *q, ShellInterface *shell, SurfaceInterface *surface, wl_resource *parentResource)
     : Resource::Private(q, shell, parentResource, &wl_shell_surface_interface, &s_interface)
-    , surface(surface)
+    , GenericShellSurface<KWayland::Server::ShellSurfaceInterface>(q, surface)
     , pingTimer(new QTimer)
 {
     pingTimer->setSingleShot(true);
     pingTimer->setInterval(1000);
 }
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct wl_shell_surface_interface ShellSurfaceInterface::Private::s_interface = {
     pongCallback,
     moveCallback,
-    resizeCallback,
+    resizeCallback<wl_shell_surface_resize>,
     setToplevelCallback,
     setTransientCallback,
     setFullscreenCallback,
     setPopupCallback,
     setMaximizedCallback,
     setTitleCallback,
-    setClassCallback
+    setAppIdCallback
 };
 #endif
 
@@ -193,10 +187,20 @@ ShellSurfaceInterface::ShellSurfaceInterface(ShellInterface *shell, SurfaceInter
 {
     Q_D();
     connect(d->pingTimer.data(), &QTimer::timeout, this, &ShellSurfaceInterface::pingTimeout);
+    auto unsetSurface = [this] {
+        Q_D();
+        d->surface = nullptr;
+    };
+    connect(parent, &Resource::unbound, this, unsetSurface);
+    connect(parent, &QObject::destroyed, this, unsetSurface);
 }
 
 ShellSurfaceInterface::~ShellSurfaceInterface() = default;
 
+void ShellSurfaceInterface::Private::commit()
+{
+}
+
 void ShellSurfaceInterface::Private::pongCallback(wl_client *client, wl_resource *resource, uint32_t serial)
 {
     auto s = cast<Private>(resource);
@@ -256,17 +260,10 @@ void ShellSurfaceInterface::requestSize(const QSize &size)
     d->client->flush();
 }
 
-void ShellSurfaceInterface::Private::moveCallback(wl_client *client, wl_resource *resource, wl_resource *seat, uint32_t serial)
+namespace {
+template <>
+Qt::Edges edgesToQtEdges(wl_shell_surface_resize edges)
 {
-    auto s = cast<Private>(resource);
-    Q_ASSERT(client == *s->client);
-    emit s->q_func()->moveRequested(SeatInterface::get(seat), serial);
-}
-
-void ShellSurfaceInterface::Private::resizeCallback(wl_client *client, wl_resource *resource, wl_resource *seat, uint32_t serial, uint32_t edges)
-{
-    auto s = cast<Private>(resource);
-    Q_ASSERT(client == *s->client);
     Qt::Edges qtEdges;
     switch (edges) {
     case WL_SHELL_SURFACE_RESIZE_TOP:
@@ -293,10 +290,14 @@ void ShellSurfaceInterface::Private::resizeCallback(wl_client *client, wl_resour
     case WL_SHELL_SURFACE_RESIZE_BOTTOM_RIGHT:
         qtEdges = Qt::BottomEdge | Qt::RightEdge;
         break;
+    case WL_SHELL_SURFACE_RESIZE_NONE:
+        break;
     default:
+        Q_UNREACHABLE();
         break;
     }
-    emit s->q_func()->resizeRequested(SeatInterface::get(seat), serial, qtEdges);
+    return qtEdges;
+}
 }
 
 void ShellSurfaceInterface::Private::setToplevelCallback(wl_client *client, wl_resource *resource)
@@ -384,6 +385,9 @@ void ShellSurfaceInterface::Private::setPopupCallback(wl_client *client, wl_reso
     emit s->q_func()->transientChanged(!s->transientFor.isNull());
     emit s->q_func()->transientOffsetChanged(s->transientOffset);
     emit s->q_func()->transientForChanged();
+    // we ignore the flags as Qt requests keyboard focus for popups
+    // if we would honor the flag this could break compositors
+    // compare QtWayland (5.6), file qwaylandwlshellsurface.cpp:208
     s->setAcceptsFocus(WL_SHELL_SURFACE_TRANSIENT_INACTIVE);
 }
 
@@ -395,40 +399,6 @@ void ShellSurfaceInterface::Private::setMaximizedCallback(wl_client *client, wl_
     s->setWindowMode(WindowMode::Maximized);
 }
 
-void ShellSurfaceInterface::Private::setTitleCallback(wl_client *client, wl_resource *resource, const char *title)
-{
-    auto s = cast<Private>(resource);
-    Q_ASSERT(client == *s->client);
-    s->setTitle(QString::fromUtf8(title));
-}
-
-void ShellSurfaceInterface::Private::setTitle(const QString &t)
-{
-    if (title == t) {
-        return;
-    }
-    title = t;
-    Q_Q(ShellSurfaceInterface);
-    emit q->titleChanged(title);
-}
-
-void ShellSurfaceInterface::Private::setClassCallback(wl_client *client, wl_resource *resource, const char *class_)
-{
-    auto s = cast<Private>(resource);
-    Q_ASSERT(client == *s->client);
-    s->setWindowClass(QByteArray(class_));
-}
-
-void ShellSurfaceInterface::Private::setWindowClass(const QByteArray &wc)
-{
-    if (windowClass == wc) {
-        return;
-    }
-    windowClass = wc;
-    Q_Q(ShellSurfaceInterface);
-    emit q->windowClassChanged(windowClass);
-}
-
 SurfaceInterface *ShellSurfaceInterface::surface() const {
     Q_D();
     return d->surface;
@@ -488,6 +458,14 @@ bool ShellSurfaceInterface::acceptsKeyboardFocus() const
     return d->acceptsKeyboardFocus;
 }
 
+void ShellSurfaceInterface::popupDone()
+{
+    Q_D();
+    if (isPopup() && d->resource) {
+        wl_shell_surface_send_popup_done(d->resource);
+    }
+}
+
 QPointer< SurfaceInterface > ShellSurfaceInterface::transientFor() const
 {
     Q_D();
