@@ -27,6 +27,7 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include "region_interface.h"
 #include "subcompositor_interface.h"
 #include "subsurface_interface_p.h"
+#include "surfacerole_p.h"
 // Qt
 #include <QListIterator>
 // Wayland
@@ -181,34 +182,68 @@ void SurfaceInterface::Private::installPointerConstraint(LockedPointerInterface
     Q_ASSERT(lockedPointer.isNull());
     Q_ASSERT(confinedPointer.isNull());
     lockedPointer = QPointer<LockedPointerInterface>(lock);
+
+    auto cleanUp = [this]() {
+        lockedPointer.clear();
+        disconnect(constrainsOneShotConnection);
+        constrainsOneShotConnection = QMetaObject::Connection();
+        disconnect(constrainsUnboundConnection);
+        constrainsUnboundConnection = QMetaObject::Connection();
+        emit q_func()->pointerConstraintsChanged();
+    };
+
     if (lock->lifeTime() == LockedPointerInterface::LifeTime::OneShot) {
         constrainsOneShotConnection = QObject::connect(lock, &LockedPointerInterface::lockedChanged, q_func(),
-            [this] {
-                if (lockedPointer.isNull()) {
+            [this, cleanUp] {
+                if (lockedPointer.isNull() || lockedPointer->isLocked()) {
                     return;
                 }
-                if (!lockedPointer->isLocked()) {
-                    lockedPointer.clear();
-                    disconnect(constrainsOneShotConnection);
-                    constrainsOneShotConnection = QMetaObject::Connection();
-                    disconnect(constrainsUnboundConnection);
-                    constrainsUnboundConnection = QMetaObject::Connection();
-                    emit q_func()->pointerConstraintsChanged();
-                }
+                cleanUp();
             }
         );
     }
     constrainsUnboundConnection = QObject::connect(lock, &LockedPointerInterface::unbound, q_func(),
-        [this] {
+        [this, cleanUp] {
             if (lockedPointer.isNull()) {
                 return;
             }
-            lockedPointer.clear();
-            disconnect(constrainsOneShotConnection);
-            constrainsOneShotConnection = QMetaObject::Connection();
-            disconnect(constrainsUnboundConnection);
-            constrainsUnboundConnection = QMetaObject::Connection();
-            emit q_func()->pointerConstraintsChanged();
+            cleanUp();
+        }
+    );
+    emit q_func()->pointerConstraintsChanged();
+}
+
+void SurfaceInterface::Private::installPointerConstraint(ConfinedPointerInterface *confinement)
+{
+    Q_ASSERT(lockedPointer.isNull());
+    Q_ASSERT(confinedPointer.isNull());
+    confinedPointer = QPointer<ConfinedPointerInterface>(confinement);
+
+    auto cleanUp = [this]() {
+        confinedPointer.clear();
+        disconnect(constrainsOneShotConnection);
+        constrainsOneShotConnection = QMetaObject::Connection();
+        disconnect(constrainsUnboundConnection);
+        constrainsUnboundConnection = QMetaObject::Connection();
+        emit q_func()->pointerConstraintsChanged();
+    };
+
+    if (confinement->lifeTime() == ConfinedPointerInterface::LifeTime::OneShot) {
+        constrainsOneShotConnection = QObject::connect(confinement, &ConfinedPointerInterface::confinedChanged, q_func(),
+            [this, cleanUp] {
+                if (confinedPointer.isNull() || confinedPointer->isConfined()) {
+                    return;
+                }
+                cleanUp();
+            }
+        );
+    }
+    constrainsUnboundConnection = QObject::connect(confinement, &ConfinedPointerInterface::unbound, q_func(),
+        [this, cleanUp] {
+            if (confinedPointer.isNull()) {
+                return;
+            }
+            cleanUp();
         }
     );
     emit q_func()->pointerConstraintsChanged();
@@ -230,45 +265,7 @@ void SurfaceInterface::Private::installIdleInhibitor(IdleInhibitorInterface *inh
     }
 }
 
-void SurfaceInterface::Private::installPointerConstraint(ConfinedPointerInterface *confinement)
-{
-    Q_ASSERT(lockedPointer.isNull());
-    Q_ASSERT(confinedPointer.isNull());
-    confinedPointer = QPointer<ConfinedPointerInterface>(confinement);
-    if (confinement->lifeTime() == ConfinedPointerInterface::LifeTime::OneShot) {
-        constrainsOneShotConnection = QObject::connect(confinement, &ConfinedPointerInterface::confinedChanged, q_func(),
-            [this] {
-                if (confinedPointer.isNull()) {
-                    return;
-                }
-                if (!confinedPointer->isConfined()) {
-                    confinedPointer.clear();
-                    disconnect(constrainsOneShotConnection);
-                    constrainsOneShotConnection = QMetaObject::Connection();
-                    disconnect(constrainsUnboundConnection);
-                    constrainsUnboundConnection = QMetaObject::Connection();
-                    emit q_func()->pointerConstraintsChanged();
-                }
-            }
-        );
-    }
-    constrainsUnboundConnection = QObject::connect(confinement, &ConfinedPointerInterface::unbound, q_func(),
-        [this] {
-            if (confinedPointer.isNull()) {
-                return;
-            }
-            confinedPointer.clear();
-            disconnect(constrainsOneShotConnection);
-            constrainsOneShotConnection = QMetaObject::Connection();
-            disconnect(constrainsUnboundConnection);
-            constrainsUnboundConnection = QMetaObject::Connection();
-            emit q_func()->pointerConstraintsChanged();
-        }
-    );
-    emit q_func()->pointerConstraintsChanged();
-}
-
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+#ifndef K_DOXYGEN
 const struct wl_surface_interface SurfaceInterface::Private::s_interface = {
     resourceDestroyedCallback,
     attachCallback,
@@ -278,7 +275,8 @@ const struct wl_surface_interface SurfaceInterface::Private::s_interface = {
     inputRegionCallback,
     commitCallback,
     bufferTransformCallback,
-    bufferScaleCallback
+    bufferScaleCallback,
+    damageBufferCallback
 };
 #endif
 
@@ -376,7 +374,9 @@ void SurfaceInterface::Private::swapStates(State *source, State *target, bool em
     // copy values
     if (bufferChanged) {
         target->buffer = buffer;
+        target->offset = source->offset;
         target->damage = source->damage;
+        target->bufferDamage = source->bufferDamage;
         target->bufferIsSet = source->bufferIsSet;
     }
     if (childrenChanged) {
@@ -443,18 +443,43 @@ void SurfaceInterface::Private::swapStates(State *source, State *target, bool em
         emit q->transformChanged(target->transform);
     }
     if (bufferChanged && emitChanged) {
-        if (!target->damage.isEmpty()) {
+        if (target->buffer && (!target->damage.isEmpty() || !target->bufferDamage.isEmpty())) {
             const QRegion windowRegion = QRegion(0, 0, q->size().width(), q->size().height());
             if (!windowRegion.isEmpty()) {
-                target->damage = windowRegion.intersected(target->damage);
+                QRegion bufferDamage;
+                if (!target->bufferDamage.isEmpty()) {
+                    typedef OutputInterface::Transform Tr;
+                    const Tr tr = target->transform;
+                    const qint32 sc = target->scale;
+                    if (tr == Tr::Rotated90 || tr == Tr::Rotated270 ||
+                            tr == Tr::Flipped90 || tr == Tr::Flipped270) {
+                        // calculate transformed + scaled buffer damage
+                        for (const auto &rect : target->bufferDamage) {
+                            const auto add = QRegion(rect.x() / sc, rect.y() / sc, rect.height() / sc, rect.width() / sc);
+                            bufferDamage = bufferDamage.united(add);
+                        }
+                    } else if (sc != 1) {
+                        // calculate scaled buffer damage
+                        for (const auto &rect : target->bufferDamage) {
+                            const auto add = QRegion(rect.x() / sc, rect.y() / sc, rect.width() / sc, rect.height() / sc);
+                            bufferDamage = bufferDamage.united(add);
+                        }
+                    } else {
+                        bufferDamage = target->bufferDamage;
+                    }
+                }
+                target->damage = windowRegion.intersected(target->damage.united(bufferDamage));
                 if (emitChanged) {
                     subSurfaceIsMapped = true;
                     trackedDamage = trackedDamage.united(target->damage);
                     emit q->damaged(target->damage);
                     // workaround for https://bugreports.qt.io/browse/QTBUG-52092
                     // if the surface is a sub-surface, but the main surface is not yet mapped, fake frame rendered
-                    if (subSurface && !subSurface->mainSurface()->buffer()) {
-                        q->frameRendered(0);
+                    if (subSurface) {
+                        const auto mainSurface = subSurface->mainSurface();
+                        if (!mainSurface || !mainSurface->buffer()) {
+                            q->frameRendered(0);
+                        }
                     }
                 }
             }
@@ -488,6 +513,7 @@ void SurfaceInterface::Private::swapStates(State *source, State *target, bool em
 
 void SurfaceInterface::Private::commit()
 {
+    Q_Q(SurfaceInterface);
     if (!subSurface.isNull() && subSurface->isSynchronized()) {
         swapStates(&pending, &subSurfacePending, false);
     } else {
@@ -505,6 +531,10 @@ void SurfaceInterface::Private::commit()
             subSurface->d_func()->commit();
         }
     }
+    if (role) {
+        role->commit();
+    }
+    emit q->committed();
 }
 
 void SurfaceInterface::Private::commitSubSurface()
@@ -524,12 +554,17 @@ void SurfaceInterface::Private::commitSubSurface()
 }
 
 void SurfaceInterface::Private::damage(const QRect &rect)
+{
+    pending.damage = pending.damage.united(rect);
+}
+
+void SurfaceInterface::Private::damageBuffer(const QRect &rect)
 {
     if (!pending.bufferIsSet || (pending.bufferIsSet && !pending.buffer)) {
         // TODO: should we send an error?
         return;
     }
-    pending.damage = pending.damage.united(rect);
+    pending.bufferDamage = pending.bufferDamage.united(rect);
 }
 
 void SurfaceInterface::Private::setScale(qint32 scale)
@@ -565,6 +600,7 @@ void SurfaceInterface::Private::attachBuffer(wl_resource *buffer, const QPoint &
         // got a null buffer, deletes content in next frame
         pending.buffer = nullptr;
         pending.damage = QRegion();
+        pending.bufferDamage = QRegion();
         return;
     }
     Q_Q(SurfaceInterface);
@@ -605,6 +641,12 @@ void SurfaceInterface::Private::damageCallback(wl_client *client, wl_resource *r
     cast<Private>(resource)->damage(QRect(x, y, width, height));
 }
 
+void SurfaceInterface::Private::damageBufferCallback(wl_client *client, wl_resource *resource, int32_t x, int32_t y, int32_t width, int32_t height)
+{
+    Q_UNUSED(client)
+    cast<Private>(resource)->damageBuffer(QRect(x, y, width, height));
+}
+
 void SurfaceInterface::Private::frameCallback(wl_client *client, wl_resource *resource, uint32_t callback)
 {
     auto s = cast<Private>(resource);
@@ -861,6 +903,35 @@ SurfaceInterface *SurfaceInterface::surfaceAt(const QPointF &position)
     return nullptr;
 }
 
+SurfaceInterface *SurfaceInterface::inputSurfaceAt(const QPointF &position)
+{
+    // TODO: Most of this is very similar to SurfaceInterface::surfaceAt
+    //       Is there a way to reduce the code duplication?
+    if (!isMapped()) {
+        return nullptr;
+    }
+    Q_D();
+    // go from top to bottom. Top most child is last in list
+    QListIterator<QPointer<SubSurfaceInterface>> it(d->current.children);
+    it.toBack();
+    while (it.hasPrevious()) {
+        const auto &current = it.previous();
+        auto surface = current->surface();
+        if (surface.isNull()) {
+            continue;
+        }
+        if (auto s = surface->inputSurfaceAt(position - current->position())) {
+            return s;
+        }
+    }
+    // check whether the geometry and input region contain the pos
+    if (!size().isEmpty() && QRectF(QPoint(0, 0), size()).contains(position) &&
+            (inputIsInfinite() || input().contains(position.toPoint()))) {
+        return this;
+    }
+    return nullptr;
+}
+
 QPointer<LockedPointerInterface> SurfaceInterface::lockedPointer() const
 {
     Q_D();
@@ -879,6 +950,18 @@ bool SurfaceInterface::inhibitsIdle() const
     return !d->idleInhibitors.isEmpty();
 }
 
+void SurfaceInterface::setDataProxy(SurfaceInterface *surface)
+{
+    Q_D();
+    d->dataProxy = surface;
+}
+
+SurfaceInterface* SurfaceInterface::dataProxy() const
+{
+    Q_D();
+    return d->dataProxy;
+}
+
 SurfaceInterface::Private *SurfaceInterface::d_func() const
 {
     return reinterpret_cast<Private*>(d.data());
