@@ -114,6 +114,7 @@ public:
     /**
      * @returns The XdgShellSurfaceInterface for the @p native resource.
      **/
+    //TODO KF6 make virtual
     XdgShellSurfaceInterface *getSurface(wl_resource *native);
 
     /**
@@ -277,6 +278,27 @@ public:
      **/
     void close();
 
+    /**
+     * @brief windowGeometry
+     * The geometry of the window within the buffer
+     *
+     * If invalid, the geometry of the bufer should be used instead
+     * @since 5.59
+     */
+    QRect windowGeometry() const;
+
+    /**
+     * @returns The minimum size for the window specified by the client.
+     * @since 5.65
+     */
+    QSize minimumSize() const;
+
+    /**
+     * @returns The maximum size for the window specified by the client.
+     * @since 5.65
+     */
+    QSize maximumSize() const;
+
 Q_SIGNALS:
     /**
      * Emitted whenever the title changes.
@@ -345,6 +367,13 @@ Q_SIGNALS:
      */
     void minSizeChanged(const QSize &size);
 
+    /**
+     * @brief windowGeometryChanged
+     * @param windowGeometry the newly changed windowGeometry
+     * @since 5.59
+     */
+    void windowGeometryChanged(const QRect &windowGeometry);
+
 protected:
     class Private;
     explicit XdgShellSurfaceInterface(Private *p);
@@ -443,6 +472,13 @@ public:
      **/
     void popupDone();
 
+    /**
+     * @brief windowGeometryChanged
+     * @param windowGeometry the newly changed geometry of the window contents within the buffer
+     * @since 5.59
+     */
+    QRect windowGeometry()const;
+
 Q_SIGNALS:
     /**
      * A configure event with @p serial got acknowledged.
@@ -461,6 +497,13 @@ Q_SIGNALS:
      */
     void grabRequested(KWayland::Server::SeatInterface *seat, quint32 serial);
 
+    /**
+     * @brief windowGeometryChanged
+     * @param windowGeometry the newly changed windowGeometry
+     * @since 5.59
+     */
+    void windowGeometryChanged(const QRect &windowGeometry);
+
 protected:
     class Private;
     explicit XdgShellPopupInterface(Private *p);
@@ -478,5 +521,6 @@ Q_DECLARE_METATYPE(KWayland::Server::XdgShellSurfaceInterface *)
 Q_DECLARE_METATYPE(KWayland::Server::XdgShellPopupInterface *)
 Q_DECLARE_METATYPE(KWayland::Server::XdgShellSurfaceInterface::State)
 Q_DECLARE_METATYPE(KWayland::Server::XdgShellSurfaceInterface::States)
+Q_DECLARE_OPERATORS_FOR_FLAGS(KWayland::Server::PositionerConstraints)
 
 #endif
