@@ -200,7 +200,7 @@ Interface::Interface() = default;
 Interface::Interface(const QXmlStreamAttributes &attributes)
     : m_name(attributes.value(QStringLiteral("name")).toString())
     , m_version(attributes.value(QStringLiteral("version")).toUInt())
-    , m_factory(Q_NULLPTR)
+    , m_factory(nullptr)
 {
     auto it = s_clientClassNameMapping.constFind(m_name);
     if (it != s_clientClassNameMapping.constEnd()) {
@@ -500,7 +500,7 @@ void Generator::startAuthorNameProcess()
         QStringLiteral("user.name")
     });
     git->setProgram(QStringLiteral("git"));
-    connect(git, static_cast<void (QProcess::*)(int)>(&QProcess::finished), this,
+    connect(git, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), this,
         [this, git] {
             QMutexLocker locker(&m_mutex);
             m_authorName = QString::fromLocal8Bit(git->readAllStandardOutput()).trimmed();
@@ -520,7 +520,7 @@ void Generator::startAuthorEmailProcess()
         QStringLiteral("user.email")
     });
     git->setProgram(QStringLiteral("git"));
-    connect(git, static_cast<void (QProcess::*)(int)>(&QProcess::finished), this,
+    connect(git, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), this,
         [this, git] {
             QMutexLocker locker(&m_mutex);
             m_authorEmail = QString::fromLocal8Bit(git->readAllStandardOutput()).trimmed();
@@ -928,7 +928,7 @@ void Generator::generateServerPrivateResourceClass(const Interface &interface)
 
 void Generator::generateServerPrivateInterfaceClass(const Interface &interface)
 {
-    *m_stream.localData() << QStringLiteral("#ifndef DOXYGEN_SHOULD_SKIP_THIS\n");
+    *m_stream.localData() << QStringLiteral("#ifndef K_DOXYGEN\n");
     *m_stream.localData() << QStringLiteral("const struct %2_interface %1::Private::s_interface = {\n").arg(interface.kwaylandServerName()).arg(interface.name());
     bool first = true;
     for (auto r: interface.requests()) {
